@@ -15,7 +15,7 @@ from . import __version__ as quilt3_version
 from .session import open_url
 from .util import get_from_config, catalog_s3_url, catalog_package_url, QuiltException, PhysicalKey, \
     fix_url, get_package_registry
-from .registry import app
+
 
 def cmd_config(catalog_url, **kwargs):
     """
@@ -57,7 +57,6 @@ def cmd_config_default_registry(default_remote_registry):
     print(f"Successfully set the default remote registry to {default_remote_registry}")
 
 
-
 def _test_url(url):
     try:
         response = requests.get(url)
@@ -67,6 +66,7 @@ def _test_url(url):
     except requests.exceptions.ConnectionError:
         return False
 
+
 def _launch_local_catalog():
     """"
     Launches a docker container to run nginx hosting
@@ -77,6 +77,7 @@ def _launch_local_catalog():
     env = dict(REGISTRY_URL="http://localhost:5000",
                S3_PROXY_URL=open_config["s3Proxy"],
                ALWAYS_REQUIRE_AUTH="false",
+               NO_DOWNLOAD="false",
                CATALOG_MODE="LOCAL",
                SSO_AUTH="DISABLED",
                PASSWORD_AUTH="ENABLED",
@@ -87,6 +88,7 @@ def _launch_local_catalog():
     command += ["-p", "3000:80", "quiltdata/catalog"]
     subprocess.Popen(command)
 
+
 def _launch_local_s3proxy():
     """"
     Launches an s3 proxy (via docker)
@@ -109,8 +111,6 @@ def _launch_local_s3proxy():
     subprocess.Popen(command)
 
 
-
-
 catalog_cmd_detailed_help = """
 Run the Quilt catalog on your machine (requires Docker). Running
 `quilt3 catalog` launches a webserver on your local machine using
@@ -118,7 +118,7 @@ Docker and a Python microservice that supplies temporary AWS
 credentials to the catalog. Temporary credentials are derived from
 your default AWS credentials (or active `AWS_PROFILE`) using
 `boto3.sts.get_session_token`. For more details about configuring and
-using AWS credentials in `boto3`, see the AWS documentation: 
+using AWS credentials in `boto3`, see the AWS documentation:
 https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html
 
 #### Previewing files in S3
@@ -134,6 +134,7 @@ sensitive data in S3 to run a private Quilt deployment. Visit
 https://quiltdata.com for more information.
 """
 
+
 def cmd_catalog(navigation_target=None, detailed_help=False):
     """
     Run the Quilt catalog locally. If navigation_targets starts with 's3://', open file view. Otherwise assume it
@@ -191,15 +192,18 @@ def cmd_catalog(navigation_target=None, detailed_help=False):
     open_url(catalog_url)
     app.run()
 
+
 def cmd_disable_telemetry():
     api._disable_telemetry()
     print("Successfully disabled telemetry.")
 
+
 def cmd_list_packages(registry):
     registry_parsed = PhysicalKey.from_url(get_package_registry(fix_url(registry)))
     for package_name in api._list_packages(registry=registry_parsed):
         print(package_name)
 
+
 def cmd_verify(name, registry, top_hash, dir, extra_files_ok):
     pkg = api.Package._browse(name, registry, top_hash)
     if pkg.verify(dir, extra_files_ok):
@@ -226,7 +230,6 @@ def create_parser():
             version=quilt3_version.strip()
     )
 
-
     subparsers = parser.add_subparsers(metavar="<command>")
     subparsers.required = True
 
@@ -265,7 +268,8 @@ def create_parser():
 
     # config-default-registry
     shorthelp = "Configure default remote registry for Quilt"
-    config_p = subparsers.add_parser("config-default-remote-registry", description=shorthelp, help=shorthelp, allow_abbrev=False)
+    config_p = subparsers.add_parser("config-default-remote-registry",
+                                     description=shorthelp, help=shorthelp, allow_abbrev=False)
     config_p.add_argument(
             "default_remote_registry",
             help="The default remote registry to use, e.g. s3://quilt-ml",
@@ -279,8 +283,8 @@ def create_parser():
     catalog_p.add_argument(
             "navigation_target",
             help="Which page in the local catalog to open. Leave blank to go to the catalog landing page, pass in an "
-             "s3 url (e.g. 's3://bucket/myfile.txt') to go to file viewer, or pass in a package name in the form "
-             "'BUCKET:USER/PKG' to go to the package viewer.",
+                 "s3 url (e.g. 's3://bucket/myfile.txt') to go to file viewer, or pass in a package name in the form "
+                 "'BUCKET:USER/PKG' to go to the package viewer.",
             type=str,
             nargs="?"
     )
@@ -293,7 +297,8 @@ def create_parser():
 
     # disable-telemetry
     shorthelp = "Disable anonymous usage metrics"
-    disable_telemetry_p = subparsers.add_parser("disable-telemetry", description=shorthelp, help=shorthelp, allow_abbrev=False)
+    disable_telemetry_p = subparsers.add_parser("disable-telemetry",
+                                                description=shorthelp, help=shorthelp, allow_abbrev=False)
     disable_telemetry_p.set_defaults(func=cmd_disable_telemetry)
 
     # install
