@@ -1,6 +1,5 @@
 import re
 from collections import OrderedDict
-from collections.abc import Mapping, Sequence, Set
 import datetime
 import json
 import os
@@ -27,7 +26,7 @@ OPEN_DATA_URL = "https://open.quiltdata.com"
 PACKAGE_NAME_FORMAT = r"([\w-]+/[\w-]+)(?:/(.+))?$"
 DISABLE_TQDM = os.getenv('QUILT_MINIMIZE_STDOUT', '').lower() == 'true'
 
-## CONFIG_TEMPLATE
+# CONFIG_TEMPLATE
 # Must contain every permitted config key, as well as their default values (which can be 'null'/None).
 # Comments are retained and added to local config, unless overridden by autoconfig via `api.config(<url>)`
 CONFIG_TEMPLATE = """
@@ -68,6 +67,7 @@ binaryApiGatewayEndpoint:
 
 """.format(BASE_PATH.as_uri() + '/packages')
 
+
 class QuiltException(Exception):
     def __init__(self, message, **kwargs):
         # We use NewError("Prefix: " + str(error)) a lot.
@@ -80,7 +80,7 @@ class QuiltException(Exception):
             setattr(self, k, v)
 
 
-class PhysicalKey(object):
+class PhysicalKey:
     __slots__ = ['bucket', 'path', 'version_id']
 
     def __init__(self, bucket, path, version_id):
@@ -146,7 +146,7 @@ class PhysicalKey(object):
             new_path = new_path.replace(os.path.sep, '/')
         # Add back a trailing '/' if the original path has it.
         if (path.endswith(os.path.sep) or
-            (os.path.altsep is not None and path.endswith(os.path.altsep))):
+                (os.path.altsep is not None and path.endswith(os.path.altsep))):
             new_path += '/'
         return cls(None, new_path, None)
 
@@ -267,7 +267,7 @@ def write_yaml(data, yaml_path, keep_backup=False):
             path.parent.mkdir(parents=True)
         with path.open('w') as config_file:
             yaml.dump(data, config_file)
-    except Exception:     #! intentionally wide catch -- reraised immediately.
+    except Exception:     # intentionally wide catch -- reraised immediately.
         if backup_path.exists():
             if path.exists():
                 path.unlink()
@@ -336,12 +336,14 @@ def validate_package_name(name):
     if not parts or parts[1]:
         raise QuiltException(f"Invalid package name: {name}.")
 
+
 def get_package_registry(path=None):
     """ Returns the package registry root for a given path """
     if path is None:
         path = get_from_config('default_local_registry')
     return path.rstrip('/') + '/.quilt'
 
+
 def configure_from_url(catalog_url):
     """ Read configuration settings from a Quilt catalog """
     config_template = read_yaml(CONFIG_TEMPLATE)
@@ -369,18 +371,20 @@ def configure_from_url(catalog_url):
 
     # Use our template + their configured values, keeping our comments.
     for key, value in new_config.items():
-        if not key in config_template:
+        if key not in config_template:
             continue
         config_template[key] = value
     write_yaml(config_template, CONFIG_PATH, keep_backup=True)
     return config_template
 
+
 def config_exists():
     """
     Returns True if a config file (config.yml) is installed.
     """
     return CONFIG_PATH.exists()
 
+
 def user_is_configured_to_custom_stack():
     """Look at the users stack to see if they have configured to their own stack. There is currently no way to
     distinguish between someone who has not configured their stack and someone who has intentionally configured
@@ -407,6 +411,7 @@ def configure_from_default():
         local_config = config_template
     return local_config
 
+
 def load_config():
     """
     Read the local config using defaults from CONFIG_TEMPLATE.
@@ -416,21 +421,25 @@ def load_config():
         local_config.update(read_yaml(CONFIG_PATH))
     return local_config
 
+
 def get_from_config(key):
     return load_config().get(key)
 
+
 def get_install_location():
     loc = get_from_config('default_install_location')
     if loc is None:
         loc = get_from_config('default_local_registry').rstrip('/')
     return loc
 
+
 def set_config_value(key, value):
     # Use local configuration (or defaults)
     local_config = load_config()
     local_config[key] = value
     write_yaml(local_config, CONFIG_PATH)
 
+
 def quiltignore_filter(paths, ignore, url_scheme):
     """Given a list of paths, filter out the paths which are captured by the
     given ignore rules.
@@ -472,6 +481,7 @@ def quiltignore_filter(paths, ignore, url_scheme):
     else:
         raise NotImplementedError
 
+
 def validate_key(key):
     """
     Verify that a file path or S3 path does not contain any '.' or '..' separators or files.
@@ -488,6 +498,7 @@ def validate_key(key):
                 f"A package entry key cannot contain a file or folder named '.' or '..' in its path."
             )
 
+
 def catalog_s3_url(catalog_url, s3_url):
     """
     Generate a URL to the Quilt catalog page for an object in S3
@@ -510,6 +521,7 @@ def catalog_s3_url(catalog_url, s3_url):
             url += f"?{urlencode(params)}"
     return url
 
+
 def catalog_package_url(catalog_url, bucket, package_name, package_timestamp="latest"):
     """
     Generate a URL to the Quilt catalog page of a package. By default will go to the latest version of the package,
@@ -522,6 +534,3 @@ def catalog_package_url(catalog_url, bucket, package_name, package_timestamp="la
     validate_package_name(package_name)
 
     return f"{catalog_url}/b/{bucket}/packages/{package_name}/tree/{package_timestamp}"
-
-
-
